
# q-expansion of newform 882.6.a.j, downloaded from the LMFDB on 29 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 882
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-4], [0], [76], [0], [-650], [-762], [-556], [2452], [2950], [674], [3024], [7730], [-17016], [21836], [-23940], [-15594], [5608], [-150], [-43784], [39178], [23570], [-17892], [38972], [6024], [-108430], [-70424], [31552], [-108282], [-72146], [-220906], [-239652], [-274172], [391154], [-339364], [29334], [71608], [-296318], [-480400], [160180], [-8984], [-182886], [-138330], [-327222], [786902], [-423098], [-1023920], [461516], [-995048], [-95568], [1044086], [1169410], [27342], [907714], [44088], [-829200], [-1319466], [-783788], [-955080], [1912730], [1026202], [-1746676], [2232120], [-1853236], [-450956], [-1602634], [20862], [2076212], [1205082], [876642], [1295930], [-3990396], [-4064518], [1672432], [3167686], [-4203884], [-342616], [3839594], [-3438942], [3894210], [1646794], [-1676592], [-566742], [-6684678], [-6913370], [4562808], [-4597598], [-1706578], [-6939158], [-2618048], [7136020], [-2173976], [-4632940], [-4566448], [5314290], [-2463140], [2799240], [1999140], [3521604], [-2606852], [-1374414], [-8783976], [-6292618], [4865816], [4463826], [8170536], [5503430], [8142512], [-2731356], [-1237326], [15975598], [1882752], [-9828042], [8212618], [-6984652], [12678884], [14032378], [-1303676], [1571100], [8341146], [-6183338], [-928966], [17913086], [-4963968], [-89526], [142396], [-10393486], [4655030], [-6721344], [12407648], [-13595802], [2568192], [20213274], [7048128], [-20411994], [-5079736], [-23389786], [-1112528], [-2008116], [30089688], [1882070], [-4882204], [-8370962], [-20209028], [13105926], [-31866706], [9947420], [6526114], [-8762376], [-6479420], [18341654], [26906494], [-15217380], [-26152036], [-10802076], [-9841668], [-27250902], [28643152], [6784920], [-30030818], [23072452], [-27143314], [16155190], [-38032252], [-22310364], [-30693006], [-15270576], [31627928], [35584234], [1859054], [22226520], [32444510], [-22587454], [53577186], [46604646], [18882120], [-44994596], [-42442128], [-38417798], [1841656], [3932626], [-28936908], [15947732], [-52632278], [33827428], [55573670], [29753616], [29144406], [13427284], [-16020234], [73262184], [-18965950], [44338758], [-63260], [47242592], [-22813482], [-54268804], [-12318850], [69495730], [-87996792], [91170732], [-27713778], [-36030504], [44444830], [-92594526], [12365628], [-40291944], [91904440], [-77283186], [41859270], [-49474460], [-41273662], [-38295692], [-81052752], [79630756], [27091380], [-31102322], [91391320], [9814508], [-23907674], [-14618414], [125274790], [-17225920], [55172118], [7279504], [28513860], [7143650], [10646216], [134871950], [61099200], [107436810], [-59998578], [-48544036], [75328024], [-66166650], [136748212], [-94276992], [52108850], [13742670], [101790522], [-44761208], [135104722], [139152916], [35915152], [-7035534], [67550120], [-86532108], [-174556320], [43120400], [-51638392], [54280374], [-30258986], [-176930700], [-148387422], [112276994], [2033576], [87100486], [121868330], [-15721836], [105368128], [-62347826], [15236860], [-157231534], [66173078], [-194213350], [-11669604], [-174246772], [-48587898], [-44840204], [-72998912], [-44921238], [85757466], [-178747660], [18418146], [-87478360], [115726058], [-225862464], [69583918], [92356818], [200703214], [198095040], [-75855528], [16369044], [-152293104], [57815770], [-112969284], [116816410], [-233963694], [183654958], [78914080], [297489120], [39827742], [83208084], [135969018], [-185188880], [-190128274], [331015704], [239210680], [300294948], [32099208], [-123157404], [239064278], [-134048814], [-2290076], [203850198], [-190611832], [9937370], [-186494854], [226881070], [-243861734], [76971866], [338335512], [279590162], [-113108446], [-18633844], [-144241690], [268277746], [-111314940], [128081630], [26201530], [-338221062], [-125346092], [-133085546], [285427728], [-119993816], [124981214], [-174063926], [16397048], [339813164], [266237554], [455621098], [82748918], [-135632534], [-41044248], [-222439120], [320219968], [-61107320], [-477584274], [-271919120], [344196058], [-137847256], [-487629274], [-338328390], [-364156524], [-156963312], [43950066], [-340638090], [92136238], [-432636388], [-377166936], [-329340888], [-227465060], [-215208538], [406393566], [14112744], [111186982], [170636156], [506911504], [-98275404], [59438374], [-62937806], [-274608122], [-338610976], [-234186070], [-240328226], [-277973908], [276541814], [-575542116], [175561716], [334131550], [489062434], [286614788], [-151235598], [-162040690], [-371964736], [-368670926], [-368729724], [-654301730], [-79003258], [-325983580], [-271165598], [219039984], [-418720762], [34484224], [268624038], [-400499260], [-644913920], [-285077680], [-298253458], [406179040], [-334007788], [-610216066], [472534764], [415021078], [-562530068], [139850138], [255699138], [350583080], [-69149236], [496054704], [383225506], [-474683894], [27426918], [592587532], [-540394856], [-321949188], [-43421096], [-63633278], [-90796626], [12458932], [335743616], [-105189538], [528182842], [-628232390], [441059364], [58292198], [849918812], [-546715850], [587130968], [171251484], [-401760524], [563837986], [-509681606], [9050554], [-390254676], [525419974], [31692136], [436257842], [-180429126], [440098284], [-489789532]]
